package com.cavaness.beer4all.security;

import java.util.Locale;
import javax.servlet.http.*;
import javax.servlet.ServletContext;
import org.apache.struts.action.*;
import org.apache.struts.util.MessageResources;
import com.cavaness.beer4all.common.view.UserView;
import com.cavaness.beer4all.framework.UserContainer;
import com.cavaness.beer4all.framework.Beer4AllBaseAction;
import com.cavaness.beer4all.util.Constants;
import com.cavaness.beer4all.service.Beer4AllService;
import com.cavaness.beer4all.common.exceptions.InvalidLoginException;

/**
 * Implements the logic to authenticate a user for the storefront application.
 */
public class LoginActionExample extends Beer4AllBaseAction {

  /**
   * Called by the controller when the a user attempts to login to the
   * storefront application.
   */
  public ActionForward execute( ActionMapping mapping,
                                ActionForm form,
                                HttpServletRequest request,
                                HttpServletResponse response )
  throws Exception{
    UserView userView = null;

    // Get the user's login name and password. They should have already
    // validated by the ActionForm.
    String username = ((LoginForm)form).getUsername();
    String password = ((LoginForm)form).getPassword();

    // Obtain the ServletContext
    ServletContext context = getServlet().getServletContext();

    // Login through the security service
    Beer4AllService serviceImpl = this.getBeer4AllService();

    Log logger = LogFactory.getLog( getClass() );
    logger.debug( "LoginAction entered" );

    try{
      // Attempt to authenticate the user
      userView = serviceImpl.authenticate(username, password);
    }catch( InvalidLoginException ex ){
      logger.error( "InvalidLoginException in the LoginAction class", ex );

      ActionErrors errors = new ActionErrors();
      ActionError newError = new ActionError( "" );
      errors.add( ActionErrors.GLOBAL_ERROR, newError );
      saveErrors( request, errors);
      return mapping.findForward( Constants.FAILURE_KEY );
    }

    UserContainer existingContainer = getUserContainer(request);
    existingContainer.setUserView( userView );

    return mapping.findForward(Constants.SUCCESS_KEY);
  }
}