package com.cavaness.beer4all.service;

import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import com.cavaness.beer4all.catalog.view.ItemView;
import com.cavaness.beer4all.catalog.view.CatalogView;
import com.cavaness.beer4all.common.view.UserView;
import com.cavaness.beer4all.common.exceptions.InvalidLoginException;
import com.cavaness.beer4all.businessobjects.*;
import com.cavaness.beer4all.common.exceptions.DatabaseException;
import com.cavaness.beer4all.common.exceptions.InvalidLoginException;
import org.apache.ojb.odmg.*;
import org.odmg.*;


public class Beer4AllOBJService implements Beer4AllService{
  Implementation odmg = null;
  Database db = null;

  public Beer4AllOBJService() throws DatabaseException {
    odmg = OJB.getInstance();
    db = odmg.newDatabase();
    initializeDatabase();
  }

  public List getFeaturedCatalogs(){
    List catalogViews = new ArrayList();

    OQLQuery query = this.createQuery( "select allCatalogs from " + Catalog.class.getName() );
    try{
      List allCatalogs = (List) query.execute();
      Iterator iter = allCatalogs.iterator();
      while (iter.hasNext() ){
        Catalog catalogBO = (Catalog)iter.next();
        CatalogView view = new CatalogView();
        view.setOid( catalogBO.getOid() );
        view.setName( catalogBO.getName() );
        catalogViews.add( view );
      }
    }catch( Exception ex ){
      ex.printStackTrace();
    }
    return catalogViews;
  }

  public List getFeaturedItems(){
    List itemViews = new ArrayList();

    OQLQuery query = this.createQuery( "select allItems from " + Item.class.getName() );
    try{
      List allItems = (List) query.execute();
      Iterator iter = allItems.iterator();
      while (iter.hasNext() ){
        Item itemBO = (Item)iter.next();
        ItemView view = new ItemView();

        view.setOid( itemBO.getOid() );
        view.setName( itemBO.getName() );
        view.setDescription( itemBO.getDescription() );
        view.setSmallUrl( itemBO.getUrl() );
        view.setPrice( itemBO.getUnitPrice() );
        itemViews.add( view );
      }
    }catch( Exception ex ){
      ex.printStackTrace();
    }
    return itemViews;
  }

  public ItemView getItemById( String itemId ) throws DatabaseException {
    String queryStr = "select item from " + Item.class.getName() + " where oid = $1";


    ItemView view = null;
    try{
      OQLQuery query = this.createQuery( queryStr );
      query.bind( itemId );

      List allItems = (List) query.execute();
      Item itemBO = (Item)allItems.get(0);

      if ( itemBO != null ){
        view = new ItemView();
        view.setOid( itemBO.getOid() );
        view.setName( itemBO.getName() );
        view.setDescription( itemBO.getDescription() );
        view.setSmallUrl( itemBO.getUrl() );
        view.setPrice( itemBO.getUnitPrice() );
      }
    }catch( Exception ex ){
      ex.printStackTrace();
      throw new DatabaseException( ex );
    }
    return view;
  }

  public List getItemsInCatalog( String catalogId ){
   return new ArrayList();
  }

 public void logout(String username){
   // Do nothing for now
 }

 public UserView authenticate(String username, String password)
  throws DatabaseException, InvalidLoginException{

/*
    String queryStr = "select item from " + ItemBO.class.getName();
        queryStr += " where id = $1";
        query.create(queryStr);
        query.bind(itemId);
*/

    UserView view = null;

    try{
      String queryStr = "select user from " + User.class.getName();
      String whereClause = " where username = $1 and password = $2";
      OQLQuery query = this.createQuery( queryStr + whereClause );
      query.bind( username );
      query.bind( password );

      List users = (List) query.execute();
      if ( users != null && users.size() > 0 ){
        User userBO = (User)users.get(0);
        view = new UserView();
        view.setOid( userBO.getOid() );
        view.setFirstName( userBO.getFirstName() );
        view.setLastName( userBO.getLastName() );
        view.setUsername( userBO.getUsername() );
      }else{
        throw new InvalidLoginException();
      }
    }catch( QueryException ex ){
      ex.printStackTrace();
      throw new DatabaseException( ex );
    }
    return view;
 }

  private void initializeDatabase() throws DatabaseException {
    try{
      db.open("repository.xml", Database.OPEN_READ_WRITE);
    }catch (ODMGException ex){
      ex.printStackTrace();
      throw new DatabaseException( ex );
    }
  }

  private void closeDatabase() throws DatabaseException {
    try{
      db.close();
    }catch (ODMGException ex){
      ex.printStackTrace();
      throw new DatabaseException( ex );
    }
  }

  private OQLQuery createQuery( String queryStr ) {
    OQLQuery query = odmg.newOQLQuery();
    try{
      query.create( queryStr );
    }catch( Exception ex ){
      ex.printStackTrace();
    }
    return query;
  }
}